from frasco.mail.provider import MailProvider
import requests
import logging


MAILGUN_API_URL = 'https://api.mailgun.net/v3'


class MailgunProvider(MailProvider):
    def send(self, msg):
        r = requests.post("%s/%s/messages.mime" % (self.options.get('mailgun_api_url', MAILGUN_API_URL), self.options['mailgun_domain']),
            auth=("api", self.options['mailgun_api_key']), data={'to': msg.send_to}, files={'message': msg.as_string()})
        r.raise_for_status()



from frasco.mail.provider import MailProvider
from flask_mail import _Mail, Connection


class SMTPProvider(MailProvider):
    def __init__(self, state, options):
        super().__init__(state, options)
        self.mail_state = _Mail(
            options.get('server', '127.0.0.1'),
            options.get('username'),
            options.get('password'),
            options.get('port', 25),
            options.get('use_tls', False),
            options.get('use_ssl', False),
            options.get('default_sender'),
            False, # debug
            options.get('max_emails'),
            False, # suppress
            options.get('ascii_attachments', False)
        )

    def connect(self):
        return Connection(self.mail_state)

    def send(self, msg):
        with self.connect() as conn:
            conn.send(msg)

    def start_bulk_connection(self):
        conn = self.connect()
        # simulate entering a with context
        # (flask-mail does not provide a way to connect otherwise)
        conn.__enter__()
        return conn

    def stop_bulk_connection(self, conn):
        conn.__exit__(None, None, None)



from frasco.ext import require_extension, get_extension_state
from flask import current_app
import os
import json


def generate_cache_service_worker(cache_name, assets, dynamic_urls=None, template_filename=None,
                            offline_fallback=None, offline_fallback_ignore_paths=None, domains=None):
    if not template_filename:
        template_filename = os.path.join(os.path.dirname(__file__), 'service-worker.js')

    if not domains:
        domains = [current_app.config['SERVER_NAME']]

    files = []
    for asset_name in assets:
        if asset_name.startswith('@'):
            files.extend(current_app.extensions.frasco_assets.env[asset_name[1:]].urls())
        else:
            files.append(asset_name)

    sw = "\n".join([
        'const CACHE_NAME = "%s";' % cache_name,
        'const CACHE_DOMAINS = %s;' % json.dumps(domains),
        'const CACHE_FILES = %s;' % json.dumps(files),
        'const CACHE_DYNAMIC_URLS = %s' % json.dumps(dynamic_urls or []),
        'const CACHE_OFFLINE_FALLBACK = "%s";' % offline_fallback,
        'const CACHE_OFFLINE_FALLBACK_IGNORE_PATHS = %s;' % json.dumps(offline_fallback_ignore_paths or [])
    ])
    
    with open(template_filename) as f:
        sw += f.read()

    return sw


def generate_cache_service_worker_response(*args, **kwargs):
    headers = {'Content-type': 'text/javascript', 'Cache-control': 'no-cache', 'Expires': '0'}
    return generate_cache_service_worker(*args, **kwargs), headers


def create_cache_service_worker_route(app, *args, **kwargs):
    require_extension('frasco_assets', app)

    @app.route(app.config.get('CACHE_SERVICE_WORKER_URL', '/cache-sw.js'))
    def cache_worker():
        state = get_extension_state('frasco_assets')
        sw = getattr(state, 'cache_service_worker', None)
        if not sw:
            sw = generate_cache_service_worker_response(*args, **kwargs)
            setattr(state, 'cache_service_worker', sw)
        return sw



from frasco.ext import get_extension_state
import sqlalchemy as sqla

from .ctx import get_locale, get_timezone, get_currency


class BabelUserModelMixin(object):
    locale = sqla.Column(sqla.String)
    timezone = sqla.Column(sqla.String)
    currency = sqla.Column(sqla.String)


def update_user_with_locale(user):
    state = get_extension_state('frasco_babel')
    setattr(user, state.options["user_locale_column"], get_locale().language)
    setattr(user, state.options["user_timezone_column"], get_timezone().zone)
    setattr(user, state.options["user_currency_column"], get_currency())



from flask import has_app_context, current_app, has_request_context, session, request
from frasco.users import user_login_context, is_user_logged_in, current_user
from frasco.utils import import_string
from frasco.ctx import ContextStack, DelayedCallsContext
from flask_rq2.job import FlaskJob
from rq.job import UNEVALUATED, dumps, Job as RQJob
from contextlib import contextmanager


prevent_circular_task_ctx = ContextStack()
synchronous_tasks = DelayedCallsContext()


@contextmanager
def prevent_circular_task(ctx_id):
    exists = ctx_id in prevent_circular_task_ctx.stack
    with prevent_circular_task_ctx(ctx_id):
        yield exists


def pack_job_args(data):
    """Traverse data and converts every object with a __taskdump__() method
    """
    if hasattr(data, "__taskdump__"):
        cls, state = data.__taskdump__()
        if not cls:
            cls = data.__class__.__module__ + "." + data.__class__.__name__
        return {"$taskobj": [cls, state]}
    if isinstance(data, (list, tuple)):
        lst = []
        for item in data:
            lst.append(pack_job_args(item))
        return lst
    if isinstance(data, dict):
        dct = {}
        for k, v in data.items():
            dct[k] = pack_job_args(v)
        return dct
    return data


def unpack_job_args(data):
    """Traverse data and transforms back objects which where dumped
    using __taskdump()
    """
    if isinstance(data, (list, tuple)):
        lst = []
        for item in data:
            lst.append(unpack_job_args(item))
        return lst
    if isinstance(data, dict):
        if "$taskobj" in data:
            cls = import_string(data["$taskobj"][0])
            return cls.__taskload__(data["$taskobj"][1])
        else:
            dct = {}
            for k, v in data.items():
                dct[k] = unpack_job_args(v)
            return dct
    return data


class FrascoJob(FlaskJob):
    @classmethod
    def create(cls, func, *args, **kwargs):
        job = super(FrascoJob, cls).create(func, *args, **kwargs)
        job.meta.setdefault('forwarded_contexts', {})['frasco.tasks.job.prevent_circular_task_ctx'] = prevent_circular_task_ctx.stack
        if is_user_logged_in():
            job.meta['current_user_id'] = current_user.id
        if has_request_context():
            job.meta.update({
                'endpoint': request.endpoint,
                'remote_addr': request.remote_addr,
                'session': dict(session)
            })
        return job

    @property
    def data(self):
        if self._data is UNEVALUATED:
            if self._func_name is UNEVALUATED:
                raise ValueError('Cannot build the job data')

            if self._instance is UNEVALUATED:
                self._instance = None

            if self._args is UNEVALUATED:
                args = ()
            else:
                args = pack_job_args(self._args)

            if self._kwargs is UNEVALUATED:
                kwargs = {}
            else:
                kwargs = pack_job_args(self._kwargs)

            job_tuple = self._func_name, self._instance, args, kwargs
            self._data = dumps(job_tuple)
        return self._data

    @data.setter
    def data(self, value):
        self._data = value
        self._func_name = UNEVALUATED
        self._instance = UNEVALUATED
        self._args = UNEVALUATED
        self._kwargs = UNEVALUATED

    def perform(self):
        app = self.load_app()
        if synchronous_tasks.calling_ctx.top or not app.config.get('RQ_ASYNC'):
            return self.perform_in_app_context(True)
        with app.app_context():
            rv = self.perform_in_app_context()
        return rv

    def perform_in_app_context(self, is_sync=False):
        clear_extended_fowarded_contexts = {}
        if not is_sync and self.meta.get('forwarded_contexts'):
            for ctx_import_str, stack in self.meta['forwarded_contexts'].items():
                import_string(ctx_import_str).stack.extend(stack)
                clear_extended_fowarded_contexts[ctx_import_str] = len(stack) # keep track using temp variable as if it runs in sync mode, stack will be modified
        try:
            current_user_id = self.meta.get('current_user_id')
            if current_user_id and not is_user_logged_in(): # user is already logged in if task is async=False
                with user_login_context(current_app.extensions.frasco_users.Model.query.get(current_user_id)):
                    rv = RQJob.perform(self)
            else:
                rv = RQJob.perform(self)
        finally:
            if clear_extended_fowarded_contexts:
                for ctx_import_str, clear_len in clear_extended_fowarded_contexts.items():
                    del import_string(ctx_import_str).stack[-clear_len:]
        return rv

    def _execute(self):
        return self.func(*unpack_job_args(self.args), **unpack_job_args(self.kwargs))



from flask.signals import Namespace


__all__ = ('user_signed_up', 'password_updated', 'reset_password_sent', 'email_validated')


_signals = Namespace()
user_signed_up = _signals.signal('user_signed_up')
password_updated = _signals.signal('user_password_updated')
reset_password_sent = _signals.signal('user_reset_password_sent')
email_validated = _signals.signal('user_email_validated')



from flask import current_app, has_request_context, request, session, _request_ctx_stack
from frasco.ext import get_extension_state, has_extension
from frasco.users import is_user_logged_in, current_user
from flask_babel import get_locale, get_timezone, refresh as refresh_babel


__all__ = ('get_locale', 'get_timezone', 'get_currency',
           'detect_locale', 'detect_timezone', 'detect_currency',
           'set_locale', 'set_timezone', 'set_currency')


_DEFAULT_CURRENCY = "USD"


def get_currency():
    """Returns the timezone that should be used for this request as
    `pytz.timezone` object.  This returns `None` if used outside of
    a request. If flask-babel was not attached to application, will
    return UTC timezone object.
    """
    ctx = _request_ctx_stack.top if has_request_context() else None
    currency = None
    babel = None
    if ctx:
        currency = getattr(ctx, 'babel_currency', None)
        babel = ctx.app.extensions.get('babel')

    if currency is None:
        if babel is None:
            currency = _DEFAULT_CURRENCY
        else:
            if getattr(babel, "currency_selector_func") is None:
                currency = babel.default_currency
            else:
                currency = babel.currency_selector_func()
                if currency is None:
                    currency = babel.default_currency
        if ctx:
            ctx.babel_currency = currency

    return currency


def _user_has_locale():
    return get_extension_state('frasco_babel').options["store_locale_in_user"] \
        and has_extension('frasco_users') and is_user_logged_in()


def detect_locale():
    state = get_extension_state('frasco_babel')
    if has_request_context() and state.options["extract_locale_from_request"]:
        if state.options["request_arg"] in request.args:
            locale = request.args[state.options["request_arg"]]
            if locale not in state.options["locales"]:
                return
            if state.options["store_request_locale_in_session"]:
                session["locale"] = locale
            return locale
    if _user_has_locale():
        locale = getattr(current_user, state.options["user_locale_column"], None)
        if locale:
            return locale
    if not has_request_context():
        return
    if state.options["store_locale_in_session"] and "locale" in session:
        return session["locale"]
    if state.options['extract_locale_from_headers']:
        return request.accept_languages.best_match(state.options["locales"])


def detect_timezone():
    state = get_extension_state('frasco_babel')
    if _user_has_locale():
        tz = getattr(current_user, state.options["user_timezone_column"], None)
        if tz:
            return tz
    if not has_request_context():
        return
    if state.options["store_locale_in_session"] and "timezone" in session:
        return session["timezone"]


def detect_currency():
    state = get_extension_state('frasco_babel')
    if _user_has_locale():
        currency = getattr(current_user, state.options["user_currency_column"], None)
        if currency:
            return currency
    if not has_request_context():
        return
    if state.options["store_locale_in_session"] and "currency" in session:
        return session["currency"]
    

def set_locale(locale, refresh=False):
    state = get_extension_state('frasco_babel')
    if _user_has_locale():
        setattr(current_user, state.options["user_locale_column"], locale)
        return
    if state.options["store_locale_in_session"]:
        session["locale"] = locale
    if refresh:
        refresh_babel()


def set_timezone(tz):
    state = get_extension_state('frasco_babel')
    if _user_has_locale():
        setattr(current_user, state.options["user_timezone_column"], tz)
        return
    if state.options["store_locale_in_session"]:
        session["timezone"] = tz


def set_currency(currency):
    state = get_extension_state('frasco_babel')
    if _user_has_locale():
        setattr(current_user, state.options["user_currency_column"], currency)
        return
    if state.options["store_locale_in_session"]:
        session["currency"] = currency



from frasco.ext import *
from flask_assets import Environment as BaseEnvironment, FlaskResolver, _webassets_cmd
from flask import Blueprint, current_app, _request_ctx_stack, has_request_context
from flask.cli import with_appcontext, cli
from flask.signals import Namespace as SignalNamespace
from ..helpers import url_for as frasco_url_for
import logging
import click
import os
import shutil


_signals = SignalNamespace()
before_build_assets = _signals.signal('before_build_assets')
after_clean_assets = _signals.signal('before_clean_assets')
auto_build_assets = _signals.signal('auto_build_assets')


class Resolver(FlaskResolver):
    """We override the convert_item_to_flask_url() to use our own url_for()"""
    def convert_item_to_flask_url(self, ctx, item, filepath=None):
        directory, rel_path, endpoint = self.split_prefix(ctx, item)

        if filepath is not None:
            filename = filepath[len(directory)+1:]
        else:
            filename = rel_path

        flask_ctx = None
        if not _request_ctx_stack.top:
            flask_ctx = ctx.environment._app.test_request_context()
            flask_ctx.push()
        try:
            url = cdn_url_for(endpoint, filename=filename)
            if url and url.startswith('http:'):
                url = url[5:]
            return url
        finally:
            if flask_ctx:
                flask_ctx.pop()


class Environment(BaseEnvironment):
    resolver_class = Resolver


class FrascoAssetsState(ExtensionState):
    def register(self, *args, **kwargs):
        return self.env.register(*args, **kwargs)


class FrascoAssets(Extension):
    name = 'frasco_assets'
    state_class = FrascoAssetsState
    prefix_extra_options = 'ASSETS_'
    defaults = {'js_packages_path': {},
                'copy_files_from_js_packages': {},
                'cdn_scheme': 'https',
                'cdn_endpoints': ['static']}

    def _init_app(self, app, state):
        state.env = Environment(app)
        state.env.debug = app.debug
        app.jinja_env.globals['url_for'] = cdn_url_for
        app.jinja_env.globals['url_for_static'] = cdn_url_for_static
        app.jinja_env.macros.register_file(os.path.join(os.path.dirname(__file__), "macros.html"), alias="frasco_assets.html")

        if state.options['copy_files_from_js_packages']:
            register_assets_builder(lambda: copy_files_from_js_packages(state.options['copy_files_from_js_packages']))

        @app.cli.command()
        @with_appcontext
        def build_all_assets():
            """Build assets from all extensions."""
            if state.options['js_packages_path']:
                register_js_packages_blueprint(app, state.options['js_packages_path'])
            before_build_assets.send()
            _webassets_cmd('build')

        if state.options['js_packages_path'] and (state.env.config["auto_build"] or app.debug):
            register_js_packages_blueprint(app, state.options['js_packages_path'])

        if state.env.config["auto_build"]:
            @app.before_first_request
            def before_first_request():
                auto_build_assets.send(self)

    @ext_stateful_method
    def register(self, state, *args, **kwargs):
        return state.register(*args, **kwargs)


class AssetsBlueprint(Blueprint):
    def __init__(self, name, import_name, **kwargs):
        kwargs.setdefault('static_url_path', '/static/vendor/%s' % name)
        kwargs.setdefault('static_folder', 'static')
        super(AssetsBlueprint, self).__init__(name, import_name, **kwargs)


def expose_package(app, name, import_name):
    bp = AssetsBlueprint(name, import_name)
    app.register_blueprint(bp)
    return bp


def register_assets_builder(func=None):
    def decorator(func):
        before_build_assets.connect(lambda sender: func(), weak=False)
        auto_build_assets.connect(lambda sender: func(), weak=False)
    if func:
        return decorator(func)
    return decorator


def register_js_packages_blueprint(app, js_packages_path):
    for name, path in js_packages_path.items():
        if name not in app.blueprints:
            bp = Blueprint(name, __name__, static_folder=os.path.abspath(path), static_url_path='/static/%s' % name)
            app.register_blueprint(bp)


def copy_files_from_js_packages(files):
    state = get_extension_state('frasco_assets')
    packages = state.options['js_packages_path']
    logger = logging.getLogger('frasco.assets')
    for src, dest in files.items():
        package, filename = src.split('/', 1)
        filename = os.path.join(packages.get(package, current_app.root_path), filename)
        if not os.path.exists(src):
            logger.warning("Cannot copy file from js packages: %s" % src)
            continue
        target = os.path.join(current_app.static_folder, dest)
        if os.path.isdir(filename) and os.path.exists(target):
            if dest.endswith('/'):
                target = os.path.join(target, os.path.basename(filename))
            else:
                logger.debug("Removing target of js package file copy: %s" % target)
                if os.path.isdir(target):
                    shutil.rmtree(target)
                else:
                    os.unlink(target)
        logger.debug("Copying js package file from '%s' to '%s'" % (filename, target))
        if os.path.isdir(filename):
            shutil.copytree(filename, target)
        else:
            if not os.path.exists(os.path.dirname(target)):
                os.makedirs(os.path.dirname(target))
            shutil.copyfile(filename, target)


def cdn_url_for(endpoint, **values):
    state = get_extension_state('frasco_assets', app=values.pop('_app', None))
    if state.options.get('cdn_domain') and is_cdn_endpoint(endpoint):
        try:
            scheme = values.pop('_scheme')
        except KeyError:
            scheme = state.options['cdn_scheme']
        urls = current_app.url_map.bind(state.options['cdn_domain'], url_scheme=scheme)
        return urls.build(endpoint, values=values, force_external=True)

    return frasco_url_for(endpoint, **values)


def cdn_url_for_static(filename, **kwargs):
    """Shortcut function for url_for('static', filename=filename)
    """
    return cdn_url_for('static', filename=filename, **kwargs)


def is_cdn_endpoint(endpoint):
    cdn_endpoints = get_extension_state('frasco_assets').options['cdn_endpoints']
    if endpoint in cdn_endpoints:
        return True
    for x in cdn_endpoints:
        if endpoint.endswith('.%s' % x):
            return True
    return False



from frasco.mail.provider import MailProvider
import requests
import logging
import base64
from email.utils import parseaddr
from flask_mail import sanitize_address


MAILJET_API_URL = 'https://api.mailjet.com/v3.1'


class MailjetProvider(MailProvider):
    def send(self, msg):
        r = requests.post("%s/send" % self.options.get('mailjet_api_url', MAILJET_API_URL),
            auth=(self.options['mailjet_api_key_public'], self.options['mailjet_api_key_private']),
            json={"Messages": [self.convert_to_mailjet_message(msg)]})
        r.raise_for_status()

    def convert_to_mailjet_message(self, msg):
        data = {
            "From": self.convert_address(msg.sender),
            "To": list([self.convert_address(addr) for addr in msg.recipients]),
            "Subject": msg.subject
        }

        if msg.cc:
            data['Cc'] = list([self.convert_address(addr) for addr in msg.cc])
        if msg.bcc:
            data['Bcc'] = list([self.convert_address(addr) for addr in msg.bcc])
        if msg.body:
            data['TextPart'] = msg.body
        if msg.html:
            data['HTMLPart'] = msg.html
        if msg.attachments:
            data['Attachments'] = []
            for attachment in msg.attachments:
                data['Attachments'].append({
                    "ContentType": attachment.content_type,
                    "Filename": attachment.filename,
                    "Base64Content": base64.b64encode(attachment.data)
                })
        if msg.extra_headers:
            data['Headers'] = msg.extra_headers
        if msg.reply_to:
            data.setdefault('Headers', {})
            data['Headers']['Reply-To'] = sanitize_address(msg.reply_to)

        return data

    def convert_address(self, addr):
        if isinstance(addr, (str, unicode)):
            addr = parseaddr(addr)
        name, addr = addr
        if name:
            return {"Email": addr, "Name": name}
        return {"Email": addr}



from frasco import current_app
from frasco.ext import get_extension_state
from suds.client import Client as SudsClient
from suds import WebFault
import xml.etree.ElementTree as ET
import requests
import datetime


EU_COUNTRIES = {
    "AT": "EUR", # Austria
    "BE": "EUR", # Belgium
    "BG": "BGN", # Bulgaria
    "DE": "EUR", # Germany
    "CY": "EUR", # Cyprus
    "CZ": "CZK", # Czech Republic
    "DK": "DKK", # Denmark
    "EE": "EUR", # Estonia
    "ES": "EUR", # Spain
    "FI": "EUR", # Finland
    "FR": "EUR", # France,
    "GR": "EUR", # Greece
    "HR": "HRK", # Croatia
    "HU": "HUF", # Hungary
    "IE": "EUR", # Ireland
    "IT": "EUR", # Italy
    "LT": "EUR", # Lithuania
    "LV": "EUR", # Latvia
    "LU": "EUR", # Luxembourg
    "MT": "EUR", # Malta
    "NL": "EUR", # Netherlands
    "PL": "PLN", # Poland
    "PT": "EUR", # Portugal
    "RO": "RON", # Romania
    "SE": "SEK", # Sweden
    "SI": "EUR", # Slovenia
    "SK": "EUR"  # Slovakia
}


KNOWN_VAT_RATES = {
    "AT": 20.0, # Austria
    "BE": 21.0, # Belgium
    "BG": 20.0, # Bulgaria
    "DE": 19.0, # Germany
    "CY": 19.0, # Cyprus
    "CZ": 21.0, # Czech Republic
    "DK": 25.0, # Denmark
    "EE": 20.0, # Estonia
    "ES": 21.0, # Spain
    "FI": 24.0, # Finland
    "FR": 20.0, # France,
    "GR": 23.0, # Greece
    "HR": 25.0, # Croatia
    "HU": 27.0, # Hungary
    "IE": 23.0, # Ireland
    "IT": 22.0, # Italy
    "LT": 21.0, # Lithuania
    "LV": 21.0, # Latvia
    "LU": 15.0, # Luxembourg
    "MT": 18.0, # Malta
    "NL": 21.0, # Netherlands
    "PL": 23.0, # Poland
    "PT": 23.0, # Portugal
    "RO": 24.0, # Romania
    "SE": 25.0, # Sweden
    "SI": 22.0, # Slovenia
    "SK": 20.0  # Slovakia
}


ECB_EUROFXREF_URL = 'http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml'
ECB_EUROFXREF_XML_NS = 'http://www.ecb.int/vocabulary/2002-08-01/eurofxref'
VIES_SOAP_WSDL_URL = 'http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl'
TIC_SOAP_WSDL_URL = 'http://ec.europa.eu/taxation_customs/tic/VatRateWebService.wsdl'


def is_eu_country(country_code):
    return country_code and country_code.upper() in EU_COUNTRIES


def should_charge_vat(country_code, eu_vat_number=None):
    return is_eu_country(country_code) and (
        get_extension_state('frasco_eu_vat').options['own_country'] == country_code or not eu_vat_number)


_exchange_rates_cache = {}
_vat_rates_cache = {}


VIESClient = None
def get_vies_soap_client():
    global VIESClient
    if not VIESClient:
        VIESClient = SudsClient(VIES_SOAP_WSDL_URL)
    return VIESClient


TICClient = None
def get_ticc_soap_client():
    global TICClient
    if not TICClient:
        TICClient = SudsClient(TIC_SOAP_WSDL_URL)
    return TICClient


class EUVATError(Exception):
    pass


def get_vat_rate(country_code, rate_type='standard'):
    country_code = country_code.upper()
    if not is_eu_country(country_code):
        raise EUVATError('Not an EU country')
    if country_code not in _vat_rates_cache:
        _vat_rates_cache[country_code] = {}
        try:
            r = get_ticc_soap_client().service.getRates(dict(memberState=country_code,
                requestDate=datetime.date.today().isoformat()))
            for rate in r.ratesResponse.rate:
                _vat_rates_cache[country_code][rate.type.lower()] = float(rate.value)
        except Exception as e:
            current_app.logger.debug(e)
            _vat_rates_cache.pop(country_code)
            return KNOWN_VAT_RATES.get(country_code)
    return _vat_rates_cache[country_code].get(rate_type.lower())


def validate_vat_number(vat_number, invalid_format_raise_error=False):
    if len(vat_number) < 3:
        if invalid_format_raise_error:
            raise EUVATError('VAT number too short')
        return False
    try:
        r = get_vies_soap_client().service.checkVat(vat_number[0:2].upper(), vat_number[2:])
        return r.valid
    except WebFault:
        pass
    return False


def fetch_exchange_rates():
    today = datetime.date.today()
    if today in _exchange_rates_cache:
        return _exchange_rates_cache[today]
    rates = {'EUR': 1.0}
    try:
        r = requests.get(ECB_EUROFXREF_URL)
        root = ET.fromstring(r.text)
        for cube in root.findall('eu:Cube/eu:Cube/eu:Cube', {'eu': ECB_EUROFXREF_XML_NS}):
            rates[cube.attrib['currency']] = float(cube.attrib['rate'])
        _exchange_rates_cache[today] = rates
    except Exception as e:
        current_app.logger.debug(e)
    return rates


def get_exchange_rate(country_code, src_currency='EUR'):
    if not is_eu_country(country_code):
        raise EUVATError('Not an EU country')
    dest_currency = EU_COUNTRIES[country_code]
    rates = fetch_exchange_rates()
    if src_currency == dest_currency:
        return 1.0
    if src_currency == 'EUR':
        return rates.get(dest_currency, 1.0)
    if src_currency not in rates:
        raise EUVATError('Can only use a currency listed in the ECB rates')
    return round(1.0 / rates[src_currency] * rates.get(dest_currency, 1.0), 5)



from frasco.ext import get_extension_state
import random
import math


def generate_first_letter_avatar_svg(name, colorstr=None, size=None):
    state = get_extension_state('frasco_users_avatars')
    size = size or state.options['flavatar_size'] or state.options["avatar_size"]
    if size and isinstance(size, int):
        size = "%spx" % size

    svg_tpl = ('<svg xmlns="http://www.w3.org/2000/svg" pointer-events="none" viewBox="0 0 100 100" '
            'width="%(w)s" height="%(h)s" style="background-color: %(bgcolor)s;">%(letter)s</svg>')

    char_svg_tpl = ('<text text-anchor="middle" y="50%%" x="50%%" dy="%(dy)s" '
                    'pointer-events="auto" fill="%(fgcolor)s" font-family="'
                    'HelveticaNeue-Light,Helvetica Neue Light,Helvetica Neue,Helvetica, Arial,Lucida Grande, sans-serif" '
                    'style="font-weight: 400; font-size: %(size)spx">%(char)s</text>')

    if not name:
        text = '?'
    else:
        text = name[0:min(state.options['flavatar_length'], len(name))]
    colors_len = len(state.options['flavatar_colors'])
    if colorstr:
        color_pos = sum([ord(c) for c in colorstr]) % colors_len
    elif ord(text[0]) < 65:
        color_pos = random.randint(0, colors_len - 1)
    else:
        color_pos = int(math.floor((ord(text[0]) - 65) % colors_len))

    color = state.options['flavatar_colors'][color_pos]
    return svg_tpl % {
        'bgcolor': color,
        'w': size,
        'h': size,
        'letter': char_svg_tpl % {
            'dy': state.options['flavatar_text_dy'],
            'fgcolor': get_contrast_color(color),
            'size': state.options['flavatar_font_size'],
            'char': text
        }
    }


def get_contrast_color(color):
    r = int(color[1:3], 16)
    g = int(color[3:5], 16)
    b = int(color[5:7], 16)
    yiq = ((r * 299) + (g * 587) + (b * 114)) / 1000
    return '#000' if yiq >= 128 else '#fff'
